#include "util.h"

void
opt_show(int ac, char **av)
{
	int i;
	fprintf(stderr, "ac=%d\n", ac);
	for(i=0; i<ac; i++) fprintf(stderr, "av[%d]='%s'\n", i, av[i]);
}

int
opt_idx(char *key, int ac, char **av)
{
	int i;

	for(i=0; i<ac; i++) if(strcmp(av[i], key) == 0) return i;
	return -1;
}

char *
opt_str(char *key, int ac, char **av, char *def)
{
	int i;

	i = opt_idx(key, ac, av);
	if(i < 0 || i+1 >= ac) return def;
	return av[i+1];
}

int
opt_int(char *key, int ac, char **av, int def)
{
	char *s;

	if((s = opt_str(key, ac, av, NULL)) == NULL) return def;
	return strtol(s, NULL, 0);
}

double
opt_double(char *key, int ac, char **av, double def)
{
	char *s;

	if((s = opt_str(key, ac, av, NULL)) == NULL) return def;
	return strtod(s, NULL);
}

void
line_to_ac_av(char *line, int *ac, char **av)
{
	int i, n = *ac;
	if(line) line = strdup(line);
	for(i=0; i<n && (av[i] = strtok(line, " \t\n")) != NULL; i++, line = NULL);
	*ac = i;
}

char *
opt_prefix(char *prev, int ac, char **av)
{
	int i, n = strlen(prev);
	for(i=0; i<ac; i++) if(strncmp(av[i], prev, n) == 0) break;
	return i < ac ? av[i] + n : NULL;
}

int
opt_ch_loop_get(char **sp, int *ch, int *v)
{
	double d;
	int ret = opt_ch_loop_get_dbl(sp, ch, &d);
	if(ret) *v = (int)d;
	return ret;
}

int
opt_ch_loop_get_dbl(char **sp, int *ch, double *v)
{
	char s_ch[8], buf[BUFSZ];
	int ret = *sp && **sp && sscanf(*sp, "%[^:]:%s", s_ch, buf) == 2;
	if(ret){
		*v = strtod(buf, sp);
		*ch = strcmp(s_ch, "all") == 0 ? -1 : strtol(s_ch, NULL, 0);
		if(**sp == ',') (*sp)++;
	}
	return ret;
}

struct stklst *
stklst_alloc(int size, struct stklst *prev)
{
	struct stklst *p;
	int sz = sizeof(*p) + size;

	if((p = malloc(sz)) == NULL) ERR("No Mem");
	p->size = size;
	p->p = p + 1;
	p->prev = prev;
	return p;
}

void
stklst_allfree(struct stklst *p)
{
	struct stklst *prev;
	for(; p; p=prev){
		prev = p->prev;
		free(p);
	}
}

int
stklst_allsize(struct stklst *p)
{
	int size = 0;
	for(; p; p=p->prev) size += p->size;
	return size;
}

static void *
arr_set(struct stklst *p, void *arr)
{
	if(p == NULL) return arr;
	arr = arr_set(p->prev, arr);
	memcpy(arr, p->p, p->size);
	return ((char*)arr) + p->size;
}

void *
stklst_to_arr(struct stklst *p)
{
	void *arr;

	if((arr = malloc(stklst_allsize(p))) == NULL) ERR("No Mem");
	arr_set(p, arr);
	stklst_allfree(p);
	return arr;
}

int
dbl_max_idx(double *p, int next_bytes, int n, double *ret_v)
{
	int idx = 0, i;
	double max_v = *p, v;
	p = (double *)( (char *)p + next_bytes );
	for(i=1; i<n; i++){
		v = *p;
		if(v > max_v){
			max_v = v;
			idx = i;
		}
		p = (double *)( (char *)p + next_bytes );
	}
	if(ret_v) *ret_v = max_v;
	return idx;
}

FILE *
fp_get(char *key, int ac, char **av, char *def_fn, char *mode, char **ret_fn)
{
	char *fn = opt_str(key, ac, av, def_fn);
	if(ret_fn) *ret_fn = fn;
	if(fn == NULL) return NULL;
	if(strcmp(fn, "-") == 0) return strcmp(mode, "r") == 0 ? stdin : stdout;
	return fopen(fn, mode);
}

void
chatt_init(struct chatt *ct, double lmt_sec)
{
	ct->lmt_sec = lmt_sec;
	ct->sec = 0;
	ct->chg_try_sec = -1;
	ct->onoff = 0;
}

int
chatt_add(struct chatt *ct, double sec, int onoff)
{
	if(onoff == ct->onoff){
		ct->chg_try_sec = -1;
		return 0;
	}
	if(ct->chg_try_sec < 0){
		ct->chg_try_sec = sec;
		return 0;
	}
	if(sec - ct->chg_try_sec < ct->lmt_sec) return 0;	

	ct->sec = ct->chg_try_sec;
	ct->onoff = onoff;
	ct->chg_try_sec = -1;
	return 1;
}

void
chatt_set(struct chatt *ct, int onoff)
{
	ct->onoff = onoff;
	ct->chg_try_sec = -1;
}


void
max_sum_init(struct max_sum *p)
{
	p->n = 0;
	p->max_i = p->min_i = -1;
}

void
max_sum_add(struct max_sum *p, double v)
{
	if(p->max_i < 0 || v > p->max_v){
		p->max_v = v;
		p->max_i = p->n;
	}
	if(p->min_i < 0 || v < p->min_v){
		p->min_v = v;
		p->min_i = p->n;
	}
	p->sum += v;
	p->n++;
}

double
max_sum_mean(struct max_sum *p)
{
	if(p->n == 0) return -1;
	return p->sum / p->n;
}

/* EOF */
