#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xlibint.h>
#include <unistd.h>
#include "x.h"

void *ximg_curr = NULL;

static Display *dpy;
static Window win;
static GC gc, gc_c;
static Pixmap pm;
static int w, h;
static XImage *pimg;

#define FORE	WhitePixel(dpy, DefaultScreen(dpy))
#define BACK	BlackPixel(dpy, DefaultScreen(dpy))

void
xinit(int ac, char **av, int *wp, int *hp)
{
	char *dpy_name = opt_str("-d", ac, av, NULL);
	w = opt_int("-w", ac, av, 640);
	h = opt_int("-h", ac, av, 480);
	if(wp) *wp = w;
	if(hp) *hp = h;

	if(!dpy_name) dpy_name = getenv("DISPLAY");
	if(!dpy_name) dpy_name = ":0";
	if((dpy = XOpenDisplay(dpy_name)) == NULL) ERR("Can't open display");

	win = XCreateSimpleWindow(dpy,
		DefaultRootWindow(dpy),
		0, 0, w, h,
		1,
		FORE, BACK);

	XMapWindow(dpy, win);

	/* ? */
	XFlush(dpy);
	sleep(opt_int("-init_wait", ac, av, 0));

	gc = XCreateGC(dpy, win, 0, 0);
	XSetForeground(dpy, gc, FORE);
	XSetBackground(dpy, gc, BACK);

	gc_c = XCreateGC(dpy, DefaultRootWindow(dpy), 0, 0);
	XSetForeground(dpy, gc_c, BACK);

	pm = XCreatePixmap(dpy, win, w, h,
			   DefaultDepth(dpy, DefaultScreen(dpy)));

	pimg = XGetImage(dpy, pm, 0, 0, w, h, AllPlanes, ZPixmap);
}

void
xfini(void)
{
	XDestroyImage(pimg);
	XFreeGC(dpy, gc);
	XFreeGC(dpy, gc_c);
	XFreePixmap(dpy, pm);
	XDestroyWindow(dpy, win);
	XCloseDisplay(dpy);
}

void
xclear(void)
{
	if(ximg_curr){
		memset(ximg_curr, 0, ximg_sz());
		return;
	}
	XFillRectangle(dpy, pm, gc_c, 0, 0, w, h);
}

static void
xbm_pset(int x, int y)
{
	unsigned char *d;

	if(x < 0 || x >= w || y < 0 ||y >= h) return;
	d = ximg_curr;
	d += ((w + 7) >> 3) * y + (x >> 3);
	*d |= 1 << (x & 7);
}

static void
xbm_line(int sx, int sy, int ex, int ey)
{
	int dx, dy, rx, ry, rxy, a, b, sum, i, j, x, y;

	if((rx = ((dx = ex - sx) < 0)) != 0) dx = -dx;
	if((ry = ((dy = ey - sy) < 0)) != 0) dy = -dy;
	rxy = (dx < dy);
	a = rxy ? dy : dx;	
	b = rxy ? dx : dy;

	sum = j = 0;
	for(i=0; i<a; i++){
		sum += b;
		if(sum > a){
			sum -= a;
			j++;
		}
		x = rxy ? j : i;
		y = rxy ? i : j;

		xbm_pset(sx + (rx ? -x : x), sy + (ry ? -y : y));
	}	
}

void
xline(int sx, int sy, int ex, int ey)
{
	if(ximg_curr){
		xbm_line(sx, sy, ex, ey);
		return;
	}
	XDrawLine(dpy, pm, gc, sx, sy, ex, ey);
}

void
xflush(void)
{
	if(ximg_curr) return;
	XCopyArea(dpy, pm, win, gc, 0, 0, w, h, 0, 0);
	XFlush(dpy);
}


int
ximg_sz(void)
{
	return ((w + 7) >> 3) * h;
}

void *
ximg_alloc(void)
{
	void *ximg;

	if((ximg = malloc(ximg_sz())) == NULL) ERR("No Mem");
	return ximg;
}

void
ximg_get(void)
{
	XImage *xi;
	int x, y, bit = 0, psz;
	unsigned char *s, *d;

	if(!ximg_curr) return;

	xi = XGetImage(dpy, pm, 0, 0, w, h, AllPlanes, ZPixmap);
	psz = xi->bits_per_pixel >> 3;
	d = ximg_curr;
	for(y=0; y<h; y++){
		for(x=0; x<w; x++){
			bit = x & 7;
			s = (unsigned char *)xi->data + y * xi->bytes_per_line + x * psz;
			if(bit == 0) *d = 0;
			*d |= (*s != 0) << bit;
			if(bit == 7) d++;
		}
		if(bit < 7) d++;
	}
	XDestroyImage(xi);
}

void
ximg_put(void)
{
	XImage *xi;
	int x, y, bit = 0, psz;
	unsigned char *s, *d, pat;

	if(!ximg_curr) return;

	xi = pimg;
	psz = xi->bits_per_pixel >> 3;
	pat = (psz < 3 ? 1 : 0xff);
	s = ximg_curr;
	for(y=0; y<h; y++){
		for(x=0; x<w; x++){
			bit = x & 7;
			d = (unsigned char *)xi->data + y * xi->bytes_per_line + x * psz;
			memset(d, ((*s >> bit) & 1) ? pat : 0, psz);
			if(bit == 7) s++;
		}
		if(bit < 7) s++;
	}
	XPutImage(dpy, win, gc, xi, 0, 0, 0, 0, w, h);
}

/* EOF */
